package com.mp3tunes;

import java.util.*;
import java.io.*;
import java.net.*;

import org.xml.sax.*;  
import org.xml.sax.helpers.*;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;

class OboeCache {
}

public class Oboe {
    public static final int OBOE_SUCCESS = 0;
    public static final int OBOE_ERROR_UNKNOWN = 1;
    public static final int OBOE_ERROR_MEMORY = 2;
    public static final int OBOE_ERROR_BAD_STATUS = 3;
    public static final int OBOE_ERROR_AUTH = 4;
    public static final int OBOE_ERROR_SYNC = 5;
    public static final int OBOE_ERROR_JSON = 6;
    public static final int OBOE_ERROR_HTTP = 7;
    public static final int OBOE_ERROR_OUT_OF_BOUNDS = 8;

    private String username, password;
    private String partner_token;
    private String session_id;
    private int sync_id;

    private Map metadata;
    private OboeCache file_cache;
    private OboeCache plist_cache;

    private String api_server = "ws.mp3tunes.com";
    private String content_server = "content.mp3tunes.com";
    private String login_server = "shop.mp3tunes.com";

    private static int errno;

    public Oboe(String partner) {
        String temp;

        partner_token = partner;

        temp = System.getProperty("api_server");
        if(temp != null)
            api_server = temp;

        temp = System.getProperty("content_server");
        if(temp != null)
            content_server = temp;

        temp = System.getProperty("login_server");
        if(temp != null)
            login_server = temp;

        metadata = new HashMap();
        file_cache = new OboeCache();
        plist_cache = new OboeCache();
    }

    private InputStream getURLAsStream(String url) {
        InputStream is;

        URL urlo;

        try {
            urlo = new URL(url);

            is = (InputStream) urlo.getContent();

            errno = OBOE_SUCCESS;
 
            return is;
        } catch (java.io.IOException e) {
            e.printStackTrace();
            errno = OBOE_ERROR_HTTP;
            return null;
        }
    }

    private String getURLAsString(String url) {
        byte[] buffer = new byte[4096];
        OutputStream os = new ByteArrayOutputStream();
        InputStream is;
        int read;

        URL urlo;

        try {
            is = getURLAsStream(url);

            if(is == null) {
                errno = OBOE_ERROR_UNKNOWN;
                return null;
            }

            while (true) {
                read = is.read(buffer);
                if (read == -1) {
                    break;
                }
 
                os.write(buffer, 0, read);
            }
 
            os.close();
            is.close();

            errno = OBOE_SUCCESS;
 
            return os.toString();
        } catch (java.io.IOException e) {
            errno = OBOE_ERROR_HTTP;
            return null;
        }
    }

    public Map parseXML(InputStream content)
    throws Exception {
        XMLReader parser;
        org.xml.sax.ContentHandler handler;
        Map map = null;

        parser = XMLReaderFactory.createXMLReader();
        handler = new OboeHandler();

        parser.setContentHandler(handler);
        parser.parse(new InputSource(content));

        map = ((OboeHandler) handler).tmap[0];

        return map;
    }

    public Map parseXMLAsLists(InputStream content)
    throws Exception {
        XMLReader parser;
        org.xml.sax.ContentHandler handler;
        Map map = null;

        parser = XMLReaderFactory.createXMLReader();
        handler = new OboeHandlerList();

        parser.setContentHandler(handler);
        parser.parse(new InputSource(content));

        map = ((OboeHandlerList) handler).tmap[0];

        return map;
    }

    public boolean login(String uname, String pword) {
        String url = "";
        InputStream content;
        Map map;

        try {
            username = uname = java.net.URLEncoder.encode(uname, "UTF-8");
            password = pword = java.net.URLEncoder.encode(pword, "UTF-8");
        } catch (java.io.UnsupportedEncodingException e) {
            errno = OBOE_ERROR_UNKNOWN;
            return false;
        }

        try {
            url = "https://" + login_server + "/api/v0/login" +
                "?username=" + uname + "&password=" + pword +
                "&output=xml&partner_token=" + partner_token;

            System.out.println(url);
            
            content = getURLAsStream(url);

            map = parseXML(content);
        } catch (Exception e) {
            try {
                url = "http://" + login_server + "/api/v0/login" +
                    "?username=" + uname + "&password=" + pword +
                    "&output=xml&partner_token=" + partner_token;

                content = getURLAsStream(url);
                
                map = parseXML(content);
            } catch (Exception ex) {
                e.printStackTrace();
                errno = OBOE_ERROR_HTTP;
                return false;
            }
        }

        if(!map.containsKey("session_id") || !map.containsKey("status") || !((String) map.get("status")).equals("1")) {
            errno = OBOE_ERROR_AUTH;
            return false;
        }

        session_id = (String) map.get("session_id");

        errno = OBOE_SUCCESS;
        return true;
    }

    public OboeTrack[] lockerData() {
        return lockerData(-1, -1);
    }

    public OboeTrack[] lockerData(int albumId, int playlistId) {
        String url;
        InputStream content;
        Map map, track;
        Iterator keys;
        Integer key;

        int count = 0;

        OboeTrack ret[];

        url = "http://" + api_server + "/api/v0/lockerData" +
            "?sid=" + session_id + "&type=track&output=xml" +
            "&partner_token=" + partner_token;
        if(playlistId != -1)
            url += "&playlist_id=" + playlistId;
        if(albumId != -1)
            url += "&album_id=" + albumId;

        System.out.println(url);

        try {
            content = getURLAsStream(url);

            map = parseXMLAsLists(content);

            if(map.containsKey("albumData"))
                map = (Map) map.get(new Integer(1));
            else
                map = (Map) map.get(new Integer(0));

            keys = map.keySet().iterator();

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                ++count;
            }

            ret = new OboeTrack[count];

            keys = map.keySet().iterator();
            count = 0;

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                
                track = (Map) map.get(key);
                ret[count] = new OboeTrack();
                ret[count].trackId = (int) Integer.parseInt((String) track.get("trackId"));
                ret[count].trackNumber = (int) Integer.parseInt((String) track.get("trackNumber"));
                ret[count].trackTitle = (String) track.get("trackTitle");
                ret[count].albumTitle = (String) track.get("albumTitle");
                ret[count].artistName = (String) track.get("artistName");
                ret[count].trackFileKey = (String) track.get("trackFileKey");
                ret[count].streamURL = (String) track.get("streamURL");
                ret[count].downloadURL = (String) track.get("downloadURL");
                ++count;
            }
        } catch (Exception e) {
            e.printStackTrace();
            errno = OBOE_ERROR_UNKNOWN;
            return null;
        }

        return ret;
    }

    public Map getLockerArtists() {
        String url;
        InputStream content;
        Map map, artist;
        Iterator keys;
        Integer key;

        Map ret = new HashMap();

        int count = 0;

        url = "http://" + api_server + "/api/v0/lockerData" +
            "?sid=" + session_id + "&type=artist" +
            "&output=xml&partner_token=" + partner_token;

        System.out.println(url);

        try {
            content = getURLAsStream(url);

            map = parseXMLAsLists(content);

            keys = map.keySet().iterator();

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                ++count;
            }

            keys = map.keySet().iterator();
            count = 0;

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                
                artist = (Map) map.get(key);
                ret.put(artist.get("artistName"), new Integer((String) artist.get("artistId")));
                ++count;
            }
        } catch (Exception e) {
            e.printStackTrace();
            errno = OBOE_ERROR_UNKNOWN;
            return null;
        }

        return ret;
    }
    public Map getLockerAlbums(int artist_id) {
        String url;
        InputStream content;
        Map map, artist;
        Iterator keys;
        Integer key;

        Map ret = new HashMap();

        int count = 0;

        url = "http://" + api_server + "/api/v0/lockerData" +
            "?sid=" + session_id + "&type=album" +
            "&output=xml&partner_token=" + partner_token;

        if(artist_id != -1)
            url += "&artist_id=" + artist_id;

        System.out.println(url);

        try {
            content = getURLAsStream(url);

            map = parseXMLAsLists(content);

            keys = map.keySet().iterator();

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                ++count;
            }

            keys = map.keySet().iterator();
            count = 0;

            while(keys.hasNext()) {
                try {
                    key = (Integer) keys.next();
                } catch (Exception e) {
                    continue;
                }
                
                artist = (Map) map.get(key);
                ret.put(artist.get("albumTitle"), new Integer((String) artist.get("albumId")));
                ++count;
            }
        } catch (Exception e) {
            e.printStackTrace();
            errno = OBOE_ERROR_UNKNOWN;
            return null;
        }

        return ret;
    }
}
